@@ -2,6 +2,8 @@ use crate::error::Error;
 use crate::math_helper;
 use crate::math_helper::float_eq;
 use crate::vector::Vector;
+use crate::vector2::Point2;
+use crate::vector3::Point3;
 
 use super::vector2::Vector2;
 use super::vector3::Vector3;
@@ -22,6 +24,12 @@ pub struct Matrix2x2 {
     pub r2: Vector2,
 }
 
+pub enum RotationAxis {
+    X,
+    Y,
+    Z,
+}
+
 pub trait Matrix {
     #[allow(non_snake_case)]
     /// Identity Matrrix NxN, where all elements Uij that i = j are `1`
@@ -38,6 +46,13 @@ pub trait Matrix {
     fn inverse(&self, delta: f32) -> Result<Self, Error>
     where
         Self: Sized;
+    /// A matrix is orthogonal if and only if transpose(A) == inverse(A)
+    fn is_orthogonal(&self) -> bool;
+    /// Uniform scale matrix is determined as Uij / i==j => i = a ^ i!=j => i = 0
+    /// | a 0 0 |
+    /// | 0 a 0 |
+    /// | 0 0 a |
+    fn scale_matrix(a: f32) -> Self;
 }
 
 impl Matrix for Matrix3x3 {
@@ -138,6 +153,19 @@ impl Matrix for Matrix3x3 {
         let traspose = cofactor_matrix.transpose();
         Ok(traspose / modulus)
     }
+
+    fn is_orthogonal(&self) -> bool {
+        let inv = self.inverse(0.0001);
+        if inv.is_ok() {
+            self.transpose() == inv.unwrap()
+        } else {
+            false
+        }
+    }
+
+    fn scale_matrix(a: f32) -> Self {
+        Matrix3x3::IDENTITY() * a
+    }
 }
 
 impl Matrix3x3 {
@@ -217,6 +245,38 @@ impl Matrix3x3 {
             elements3[2],
         )
     }
+
+    pub fn scale_matrix_non_uniform(a: f32, b: f32, c: f32) -> Self {
+        Matrix3x3::new_idx(a, 0f32, 0f32, 0f32, b, 0f32, 0f32, 0f32, c)
+    }
+
+    /// 2D rotation Matrix by angle teta (radians) over RotationAxis { X, Y, Z }
+    /// Matrix (Z) 3x3:
+    ///       | cosθ −sinθ 0 |
+    /// Rz(θ)=| sinθ  cosθ 0 |
+    ///       |    0     0 1 |
+    /// Matrix (X) 3x3:
+    ///       | 1    0     0 |
+    /// Rx(θ)=| 0 cosθ −sinθ |
+    ///       | 0 sinθ  cosθ |
+    /// Matrix (Y) 3x3:
+    ///       | cosθ 0 -sinθ |
+    /// Ry(θ)=|    0 1     0 |
+    ///       | sinθ 0  cosθ |
+    pub fn rotation_2d(teta: f32, rotation_axis: RotationAxis) -> Self {
+        let (sin, cos) = teta.sin_cos();
+        match rotation_axis {
+            RotationAxis::X => {
+                Matrix3x3::new_idx(1f32, 0f32, 0f32, 0f32, cos, -sin, 0f32, sin, cos)
+            }
+            RotationAxis::Y => {
+                Matrix3x3::new_idx(cos, 0f32, -sin, 0f32, 1f32, 0f32, sin, 0f32, cos)
+            }
+            RotationAxis::Z => {
+                Matrix3x3::new_idx(cos, -sin, 0f32, sin, cos, 0f32, 0f32, 0f32, 1f32)
+            }
+        }
+    }
 }
 
 impl Matrix for Matrix2x2 {
@@ -292,6 +352,19 @@ impl Matrix for Matrix2x2 {
             ))
         }
     }
+
+    fn is_orthogonal(&self) -> bool {
+        let inv = self.inverse(0.0001);
+        if inv.is_ok() {
+            self.transpose() == inv.unwrap()
+        } else {
+            false
+        }
+    }
+
+    fn scale_matrix(a: f32) -> Self {
+        Matrix2x2::IDENTITY() * a
+    }
 }
 
 impl Matrix2x2 {
@@ -307,6 +380,19 @@ impl Matrix2x2 {
             r2: Vector2::new(n3, n4),
         }
     }
+
+    pub fn scale_matrix_non_uniform(a: f32, b: f32) -> Self {
+        Matrix2x2::new_idx(a, 0f32, 0f32, b)
+    }
+
+    /// 2D rotation Matrix by angle teta (radians)
+    /// For matrix 2x2:
+    /// P′= | cosθ − sinθ | P
+    ///     | sinθ + cosθ |
+    pub fn rotation_2d(teta: f32) -> Self {
+        let (sin, cos) = teta.sin_cos();
+        Matrix2x2::new_idx(cos, -sin, sin, cos)
+    }
 }
 
 impl ops::Add for Matrix3x3 {
@@ -363,7 +449,7 @@ impl ops::Mul<Vector3> for Matrix3x3 {
     type Output = Vector3;
 
     ///Implements the transform matrix of a vector 3 into another vector 3.
-    /// The order should be matrix * vector becaus of 3x3 * 3x1 = 3x1
+    /// The order should be matrix * vector   because of 3x3 * 3x1 = 3x1
     fn mul(self, vec: Vector3) -> Vector3 {
         Vector3 {
             x: &self.r1 * &vec,
@@ -377,7 +463,7 @@ impl ops::Mul<&Vector3> for Matrix3x3 {
     type Output = Vector3;
 
     ///Implements the transform matrix of a vector 3 into another vector 3.
-    /// The order should be matrix * vector becaus of 3x3 * 3x1 = 3x1
+    /// The order should be matrix * vector   because of 3x3 * 3x1 = 3x1
     fn mul(self, vec: &Vector3) -> Vector3 {
         Vector3 {
             x: &self.r1 * vec,
@@ -391,7 +477,7 @@ impl ops::Mul<&Vector3> for &Matrix3x3 {
     type Output = Vector3;
 
     ///Implements the transform matrix of a vector 3 into another vector 3.
-    /// The order should be matrix * vector becaus of 3x3 * 3x1 = 3x1
+    /// The order should be matrix * vector   because of 3x3 * 3x1 = 3x1
     fn mul(self, vec: &Vector3) -> Vector3 {
         Vector3 {
             x: &self.r1 * vec,
@@ -401,6 +487,34 @@ impl ops::Mul<&Vector3> for &Matrix3x3 {
     }
 }
 
+impl ops::Mul<&Point3> for Matrix3x3 {
+    type Output = Point3;
+
+    ///Implements the transform matrix of a point 3 into another point 3.
+    /// The order should be matrix * point  because of 3x3 * 3x1 = 3x1
+    fn mul(self, vec: &Point3) -> Point3 {
+        Point3 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
+            z: &self.r3 * vec,
+        }
+    }
+}
+
+impl ops::Mul<&Point3> for &Matrix3x3 {
+    type Output = Point3;
+
+    ///Implements the transform matrix of a point 3 into another point 3.
+    /// The order should be matrix * point  because of 3x3 * 3x1 = 3x1
+    fn mul(self, vec: &Point3) -> Point3 {
+        Point3 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
+            z: &self.r3 * vec,
+        }
+    }
+}
+
 impl ops::Mul<f32> for Matrix3x3 {
     type Output = Matrix3x3;
 
@@ -453,6 +567,31 @@ impl ops::Mul<&Matrix3x3> for f32 {
     }
 }
 
+impl ops::Mul<Matrix3x3> for Matrix3x3 {
+    type Output = Matrix3x3;
+
+    ///Implements the Matrix 2x2 '*' trait for `Matrix3x3 * Matrix3x3`
+    fn mul(self, m: Matrix3x3) -> Matrix3x3 {
+        Matrix3x3::new(
+            Vector3::new(
+                self.r1.x * m.r1.x + self.r1.y * m.r2.x + self.r1.z * m.r3.x,
+                self.r1.x * m.r1.y + self.r1.y * m.r2.y + self.r1.z * m.r3.y,
+                self.r1.x * m.r1.z + self.r1.y * m.r2.z + self.r1.z * m.r3.z,
+            ),
+            Vector3::new(
+                self.r2.x * m.r1.x + self.r2.y * m.r2.x + self.r2.z * m.r3.x,
+                self.r2.x * m.r1.y + self.r2.y * m.r2.y + self.r2.z * m.r3.y,
+                self.r2.x * m.r1.z + self.r2.y * m.r2.z + self.r2.z * m.r3.z,
+            ),
+            Vector3::new(
+                self.r3.x * m.r1.x + self.r3.y * m.r2.x + self.r3.z * m.r3.x,
+                self.r3.x * m.r1.y + self.r3.y * m.r2.y + self.r3.z * m.r3.y,
+                self.r3.x * m.r1.z + self.r3.y * m.r2.z + self.r3.z * m.r3.z,
+            ),
+        )
+    }
+}
+
 impl ops::Div<f32> for Matrix3x3 {
     type Output = Matrix3x3;
 
@@ -551,6 +690,24 @@ impl ops::Mul<&Matrix2x2> for f32 {
     }
 }
 
+impl ops::Mul<Matrix2x2> for Matrix2x2 {
+    type Output = Matrix2x2;
+
+    ///Implements the Matrix 2x2 '*' trait for `Matrix2x2 * Matrix2x2`
+    fn mul(self, m: Matrix2x2) -> Matrix2x2 {
+        Matrix2x2::new(
+            Vector2::new(
+                self.r1.x * m.r1.x + self.r1.y * m.r2.x,
+                self.r1.x * m.r1.y + self.r1.y * m.r2.y,
+            ),
+            Vector2::new(
+                self.r2.x * m.r1.x + self.r2.y * m.r2.x,
+                self.r2.x * m.r1.y + self.r2.y * m.r2.y,
+            ),
+        )
+    }
+}
+
 impl ops::Div<f32> for Matrix2x2 {
     type Output = Matrix2x2;
 
@@ -579,7 +736,7 @@ impl ops::Mul<Vector2> for Matrix2x2 {
     type Output = Vector2;
 
     ///Implements the transform matrix of a vector 2 into another vector 2.
-    /// The order should be matrix * vector becaus of 2x2 * 2x1 = 2x1
+    /// The order should be matrix * vector   because of 2x2 * 2x1 = 2x1
     fn mul(self, vec: Vector2) -> Vector2 {
         Vector2 {
             x: &self.r1 * &vec,
@@ -592,7 +749,7 @@ impl ops::Mul<&Vector2> for Matrix2x2 {
     type Output = Vector2;
 
     ///Implements the transform matrix of a vector 2 into another vector 2.
-    /// The order should be matrix * &vector becaus of 2x2 * 2x1 = 2x1
+    /// The order should be matrix * &vector   because of 2x2 * 2x1 = 2x1
     fn mul(self, vec: &Vector2) -> Vector2 {
         Vector2 {
             x: &self.r1 * vec,
@@ -605,7 +762,7 @@ impl ops::Mul<&Vector2> for &Matrix2x2 {
     type Output = Vector2;
 
     ///Implements the transform matrix of a vector 2 into another vector 2.
-    /// The order should be matrix * vector becaus of 2x2 * 2x1 = 2x1
+    /// The order should be matrix * vector   because of 2x2 * 2x1 = 2x1
     fn mul(self, vec: &Vector2) -> Vector2 {
         Vector2 {
             x: &self.r1 * vec,
@@ -614,6 +771,32 @@ impl ops::Mul<&Vector2> for &Matrix2x2 {
     }
 }
 
+impl ops::Mul<&Point2> for Matrix2x2 {
+    type Output = Point2;
+
+    ///Implements the transform matrix of a vector 2 into another vector 2.
+    /// The order should be matrix * &point   because of 2x2 * 2x1 = 2x1
+    fn mul(self, vec: &Point2) -> Point2 {
+        Point2 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
+        }
+    }
+}
+
+impl ops::Mul<&Point2> for &Matrix2x2 {
+    type Output = Point2;
+
+    ///Implements the transform matrix of a point 2 into another point 2.
+    /// The order should be matrix * point   because of 2x2 * 2x1 = 2x1
+    fn mul(self, vec: &Point2) -> Point2 {
+        Point2 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests_matrix3x3 {
     use super::*;
@@ -653,6 +836,15 @@ mod tests_matrix3x3 {
         assert_eq!(expected, actual);
     }
 
+    #[test]
+    fn point_transform_by_matrix() {
+        let point = Point3::new(1f32, 2f32, 3f32);
+        let matrix = Matrix3x3::new_idx(1f32, 2f32, 3f32, 4f32, 5f32, 6f32, 7f32, 8f32, 9f32);
+        let actual = &matrix * &point;
+        let expected = Point3::new(14f32, 32f32, 50f32);
+        assert_eq!(expected, actual);
+    }
+
     #[test]
     fn vector_transform_by_matrix_by_ref() {
         let vec = Vector3::new(1f32, 2f32, 3f32);
@@ -720,6 +912,123 @@ mod tests_matrix3x3 {
         let identity_4 = 4f32 * &Matrix3x3::IDENTITY();
         assert_eq!(64f32, identity_4.det());
     }
+
+    #[test]
+    fn uniform_scale() {
+        let matrix = Matrix3x3::scale_matrix(4f32);
+        let expected = Matrix3x3::new_idx(4f32, 0f32, 0f32, 0f32, 4f32, 0f32, 0f32, 0f32, 4f32);
+
+        assert_eq!(matrix, expected);
+    }
+
+    #[test]
+    fn non_uniform_scale() {
+        let matrix = Matrix3x3::scale_matrix_non_uniform(4f32, 5f32, 6f32);
+        let expected = Matrix3x3::new_idx(4f32, 0f32, 0f32, 0f32, 5f32, 0f32, 0f32, 0f32, 6f32);
+
+        assert_eq!(matrix, expected);
+    }
+
+    #[test]
+    fn rotation_2d_z() {
+        let pi_quarters = 0.78539816f32;
+        let rotation_matrix = Matrix3x3::rotation_2d(pi_quarters, RotationAxis::Z);
+        let expect = Matrix3x3 {
+            r1: Vector3 {
+                x: 0.70710677,
+                y: -0.70710677,
+                z: 0.0,
+            },
+            r2: Vector3 {
+                x: 0.70710677,
+                y: 0.70710677,
+                z: 0.0,
+            },
+            r3: Vector3 {
+                x: 0.0,
+                y: 0.0,
+                z: 1.0,
+            },
+        };
+
+        assert_eq!(rotation_matrix, expect);
+    }
+
+    #[test]
+    fn rotation_2d_x() {
+        let pi_quarters = 0.78539816f32;
+        let rotation_matrix = Matrix3x3::rotation_2d(pi_quarters, RotationAxis::X);
+        let expect = Matrix3x3 {
+            r1: Vector3 {
+                x: 1.0,
+                y: 0.0,
+                z: 0.0,
+            },
+            r2: Vector3 {
+                x: 0.0,
+                y: 0.70710677,
+                z: -0.70710677,
+            },
+            r3: Vector3 {
+                x: 0.0,
+                y: 0.70710677,
+                z: 0.70710677,
+            },
+        };
+
+        assert_eq!(rotation_matrix, expect);
+    }
+
+    #[test]
+    fn rotation_2d_y() {
+        let pi_quarters = 0.78539816f32;
+        let rotation_matrix = Matrix3x3::rotation_2d(pi_quarters, RotationAxis::Y);
+        let expect = Matrix3x3 {
+            r1: Vector3 {
+                x: 0.70710677,
+                y: 0.0,
+                z: -0.70710677,
+            },
+            r2: Vector3 {
+                x: 0.0,
+                y: 1.0,
+                z: 0.0,
+            },
+            r3: Vector3 {
+                x: 0.70710677,
+                y: 0.0,
+                z: 0.70710677,
+            },
+        };
+
+        assert_eq!(rotation_matrix, expect);
+    }
+
+    #[test]
+    fn matrix_multiplication() {
+        let pi_quarters = 0.78539816f32;
+        let rotation_matrix = Matrix3x3::rotation_2d(pi_quarters, RotationAxis::Z);
+        let matrix = Matrix3x3::new_idx(1f32, 2f32, 3f32, 4f32, 5f32, 6f32, 7f32, 8f32, 9f32);
+        let expect = Matrix3x3 {
+            r1: Vector3 {
+                x: -2.1213202,
+                y: -2.1213202,
+                z: -2.1213202,
+            },
+            r2: Vector3 {
+                x: 3.535534,
+                y: 4.9497476,
+                z: 6.3639607,
+            },
+            r3: Vector3 {
+                x: 7.0,
+                y: 8.0,
+                z: 9.0,
+            },
+        };
+
+        assert_eq!(rotation_matrix * matrix, expect);
+    }
 }
 
 #[cfg(test)]
@@ -814,6 +1123,15 @@ mod tests_matrix2x2 {
         assert_eq!(expected, actual);
     }
 
+    #[test]
+    fn point_transform_by_matrix() {
+        let point = Point2::new(1f32, 2f32);
+        let matrix = Matrix2x2::new_idx(1f32, 2f32, 3f32, 4f32);
+        let actual = &matrix * &point;
+        let expected = Point2::new(5f32, 11f32);
+        assert_eq!(expected, actual);
+    }
+
     #[test]
     fn vector_transform_by_matrix_by_ref_vec_only() {
         let vec = Vector2::new(1f32, 2f32);
@@ -850,6 +1168,51 @@ mod tests_matrix2x2 {
         let expected = Matrix2x2::new_idx(1f32, 3f32, 2f32, 4f32);
         assert_eq!(expected, matrix.transpose());
     }
+
+    #[test]
+    fn orthogonal() {
+        let matrix = Matrix2x2::new_idx(1.0, 0.0, 0.0, -1.0);
+        assert!(matrix.is_orthogonal())
+    }
+
+    #[test]
+    fn not_orthogonal() {
+        let matrix = Matrix2x2::new_idx(1.0, 1.0, 1.0, 1.0);
+        assert!(!matrix.is_orthogonal())
+    }
+
+    #[test]
+    fn uniform_scale() {
+        let matrix = Matrix2x2::scale_matrix(4f32);
+        let expected = Matrix2x2::new_idx(4f32, 0f32, 0f32, 4f32);
+
+        assert_eq!(matrix, expected);
+    }
+
+    #[test]
+    fn non_uniform_scale() {
+        let matrix = Matrix2x2::scale_matrix_non_uniform(4f32, 5f32);
+        let expected = Matrix2x2::new_idx(4f32, 0f32, 0f32, 5f32);
+
+        assert_eq!(matrix, expected);
+    }
+
+    #[test]
+    fn rotation_2d() {
+        let pi_quarters = 0.78539816f32;
+        let matrix_rot_2d = Matrix2x2::rotation_2d(pi_quarters);
+        let expected = Matrix2x2::new_idx(0.70710677, -0.70710677, 0.70710677, 0.70710677);
+        assert_eq!(matrix_rot_2d, expected)
+    }
+
+    #[test]
+    fn matrix_mul() {
+        let pi_quarters = 0.78539816f32;
+        let matrix_rot_2d = Matrix2x2::rotation_2d(pi_quarters);
+        let matrix = Matrix2x2::new_idx(1f32, 2f32, 3f32, 4f32);
+        let expected = Matrix2x2::new_idx(-1.4142134, -1.4142135, 2.828427, 4.2426405);
+        assert_eq!(matrix_rot_2d * matrix, expected)
+    }
 }
 
 #[cfg(test)]
@@ -914,6 +1277,28 @@ mod tests_matrix3x3_inverse_functions {
 
         assert_eq!(matrix.modulus(), -2f32);
     }
+
+    #[test]
+    fn orthogonal() {
+        let matrix = Matrix3x3::new_idx(
+            2f32 / 3f32,
+            1f32 / 3f32,
+            2f32 / 3f32,
+            -2f32 / 3f32,
+            2f32 / 3f32,
+            1f32 / 3f32,
+            1f32 / 3f32,
+            2f32 / 3f32,
+            -2f32 / 3f32,
+        );
+        assert!(matrix.is_orthogonal())
+    }
+
+    #[test]
+    fn not_orthogonal() {
+        let matrix = Matrix3x3::new_idx(1f32, 1f32, 1f32, 1f32, 1f32, 1f32, 1f32, 1f32, 1f32);
+        assert!(!matrix.is_orthogonal())
+    }
 }
 
 #[cfg(test)]
